package Net::DNS::Keyset;

#
# $Id: KeysetStub.pm 1300 2015-01-20 09:07:16Z willem $
#
use vars qw($VERSION);
$VERSION = (qw$LastChangedRevision: 1300 $)[1];


use strict;
use warnings;
use Carp;
use base qw(Net::DNS::SEC::Keyset);

=head1 NAME

Net::DNS::Keyset - redirection stub

=cut


carp "\n\tWARNING: Net::DNS::Keyset has moved to Net::DNS::SEC::Keyset
\tPlease correct package reference";


1;
__END__


=head1 SYNOPSIS

    use Net::DNS::Keyset;		## deprecated ##

    use Net::DNS::SEC::Keyset;


=head1 DESCRIPTION

This is a redirection stub for Net::DNS::Keyset which will be removed
in a future release. The package now resides at Net::DNS::SEC::Keyset


=head1 COPYRIGHT

Copyright (c)2015 Dick Franks

All rights reserved.

This program is free software; you may redistribute it and/or
modify it under the same terms as Perl itself.


=head1 SEE ALSO

L<Net::DNS::SEC>, L<Net::DNS::SEC::Keyset>

=cut

